fromBase: Convert a base representation of a number into an integer

View source: R/RcppExports.R

fromBaseR Documentation

Convert a base representation of a number into an integer

Description

Converts a base representation of a number into an integer.

Usage

fromBase(x, base = 2L)

Arguments

x

vector of the positive integer coefficients representing the number in a base that is base.

base

positive integer representing the base.

Value

The function returns a positive integer that is a conversion from base under the given coefficients x.

Examples

fromBase(c(1, 2, 0, 2, 3), 5)

mnorm documentation built on April 14, 2026, 5:07 p.m.

Related to fromBase in mnorm...