| qnormFast | R Documentation |
Calculate the quantile of a normal distribution using one of the available methods.
qnormFast(
p,
mean = 0,
sd = 1L,
method = "Voutier",
is_validation = TRUE,
n_cores = 1L
)
p |
numeric vector of values between 0 and 1 representing the levels of the quantiles. |
mean |
numeric value representing the expectation of a normal distribution. |
sd |
positive numeric value representing the standard deviation of a normal distribution. |
method |
character representing the method to be used for the quantile calculation. Available options are "Voutier" (default) and "Shore". |
is_validation |
logical value indicating whether input
arguments should be validated. Set it to |
n_cores |
positive integer representing the number of CPU cores
used for parallel computing. Currently it is not recommended to set
|
If method = "Voutier", then the method of P. Voutier (2010)
is used, whose maximum absolute error is about 0.000025.
If method = "Shore", then the approach proposed
by H. Shore (1982) is applied, whose maximum absolute error is about
0.026 for the quantiles at level between 0.0001
and 0.9999.
The function returns a vector of p-level quantiles of a
normal distribution with the mean equal to mean and the standard
deviation equal to sd.
H. Shore (1982) <doi:10.2307/2347972>
P. Voutier (2010) <doi:10.48550/arXiv.1002.0567>
qnormFast(c(0.1, 0.9), mean = 1, sd = 2)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.