toBase: Convert an integer value to another base

View source: R/RcppExports.R

toBaseR Documentation

Convert an integer value to another base

Description

Converts an integer value to another base.

Usage

toBase(x, base = 2L)

Arguments

x

positive integer representing the number to convert.

base

positive integer representing the base.

Value

The function returns a numeric vector containing the representation of x in a base given in base.

Examples

toBase(888, 5)

mnorm documentation built on April 14, 2026, 5:07 p.m.

Related to toBase in mnorm...