| fromBase | R Documentation |
Converts a base representation of a number into an integer.
fromBase(x, base = 2L)
x |
vector of the positive integer coefficients representing the number
in a base that is |
base |
positive integer representing the base. |
The function returns a positive integer that is a
conversion from base under the given coefficients x.
fromBase(c(1, 2, 0, 2, 3), 5)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.